# Copyright (c) 2003-2004 Chalmers University of Technology
# Authors: Jean-Marc Orliaguet <jmo@ita.chalmers.se>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

__author__ = "Jean-Marc Orliaguet <jmo@ita.chalmers.se>"

"""
  PortalThemesTool
"""

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo, Unauthorized

from OFS.Folder import Folder

from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.Expression import Expression
from Products.CMFCore.utils import getToolByName

import QuickImporter
from ThemeFolder import ThemeFolder
from Products.CMFCore.PortalFolder import PortalFolder

from Products.CPSSkins.cpsskins_utils import getFreeId, getFreeTitle

CPSSKINS_THEME_COOKIE_NAME = 'cpsskins_theme'
CPSSKINS_RAMCACHE_ID = 'cpsskins-templetcache'
CPSSKINS_RENDER_METHOD_ID = 'cpsskins_render'

class PortalThemesTool(ThemeFolder, ActionProviderBase):
    """
    Portal Themes Tool
    """

    id = 'portal_themes'
    meta_type = 'Portal Themes Tool'


    _actions = [ActionInformation(id='configThemes'
                            , title='_action_themes_reconfig_'
                            , description='Configure Portal Themes'
                            , action=Expression(
            text='string: ${portal_url}/cpsskins_themes_reconfig_form')
                            , permissions=('View',)
                            , category='global'
                            , condition='python: member and portal.portal_membership.checkPermission(\'Manage Themes\', portal.portal_themes)'
                            , visible=1
                             ),
                             ] 


    security = ClassSecurityInfo()

    manage_options = ( Folder.manage_options[0:1]
                     + ( {'label': 'Default theme', 'action': 'manage_selectDefaultTheme'}, )
                     + ( {'label': 'Rebuild', 'action': 'manage_themesRebuild'}, )
                     + ( {'label' : 'Overview', 'action' : 'manage_overview' }, )
                     + ActionProviderBase.manage_options 
                     )

    security = ClassSecurityInfo()

    security.declareProtected('Manage Themes', 'manage_themesRebuild')
    manage_themesRebuild = DTMLFile('zmi/manage_themesRebuild', globals())

    security.declareProtected('Manage Themes', 'manage_selectDefaultTheme')
    manage_selectDefaultTheme = DTMLFile('zmi/manage_selectDefaultTheme', globals())


    def __init__(self):
        PortalFolder.__init__(self, self.id)

    #
    #   ActionProvider interface
    #
    security.declarePublic('listActions')
    def listActions(self, info=None):
        """
        Return actions provided via tool.
        """
        return self._actions 

    #
    # ZMI
    #
    security.declareProtected('Manage Themes', 'manage_overview')
    manage_overview = DTMLFile('zmi/explainPortalThemesTool', globals())

    #
    # Public API
    #

    security.declarePublic('getThemeCookieID')
    def getThemeCookieID(self):
        """ Gets the cookie ID used to set themes """
        return CPSSKINS_THEME_COOKIE_NAME


    security.declarePublic('getPortalThemeRoot')
    def getPortalThemeRoot(self, object=None):
        """ Gets the portal theme root container of a given object """

        if object is None:
            return
        rurl = object.absolute_url(relative=1)
        path = rurl.split('/')
        path_length = len(path)
    
        for p in range(0,path_length):
            if path[p] == 'portal_themes' and p < path_length - 1:
                theme_name = path[p+1]
                theme_container = self.getThemeContainer(theme=theme_name)
                return theme_container     
        return


    security.declarePublic('findStylesByContext')
    def findStylesByContext(self, category=None, object=None ):
        """ Gets the list of available styles """

        style = {}
        title_list = []
        object_list = []
        if object is None:
             return

        themeroot = self.getPortalThemeRoot(object)
        if themeroot is None:
             return

        styles_dir = getattr(themeroot, 'styles', None)
        if styles_dir is None:
            return

        for obj in styles_dir.objectValues():
            if getattr(obj, 'meta_type', None) == category:
                title_list.append(obj.title)
                object_list.append(obj)

        style['title'] = title_list
        style['object'] = object_list
        return style

    security.declarePublic('listPalettes')
    def listPalettes(self, category=None, object=None ):
        """ Gets the list of available palettes for a given object"""

        palette = {}
        title_list = []
        object_list = []
        if object is None:
             return

        themeroot = self.getPortalThemeRoot(object)
        if themeroot is None:
             return

        palettes_dir = getattr(themeroot, 'palettes', None)
        if palettes_dir is None:
            return

        for obj in palettes_dir.objectValues():
            if getattr(obj.aq_explicit, 'meta_type', None) == category:
                title_list.append(obj.title)
                object_list.append(obj)

        palette['title'] = title_list
        palette['object'] = object_list
        return palette


    security.declarePublic('listStyleTypes')
    def listStyleTypes(self):
        """ Gets the list of style types"""

        list = []

        for ti in self.portal_types.listTypeInfo():
           try:
              isstyle = ti.getActionById('isstyle', None)
           except:
              continue
           else:
              if isstyle:
                  list.append(ti)

        return list


    security.declarePublic('getPageBlocks')
    def getPageBlocks(self, theme=None):
        """ returns a list of page blocks sorted by ypos"""

        theme_container =  self.getEffectiveThemeContainer(theme=theme)
        if theme_container is None:
            return None

        return theme_container.getPageBlocks()

    
    security.declarePublic('getThemeContainer')
    def getThemeContainer(self, theme=None, parent=None ):
        """Gets the themes container.
           - theme = 'printable' | 'default' | ... 
           - theme = None : will return the first available theme
           - parent = 1 : returns the 'themes' folder
        """

        theme_container = getToolByName(self, 'portal_themes')

        if parent:
            return theme_container

        themes = theme_container.objectIds()
        if theme is not None and theme in themes:
            return getattr(theme_container, theme, None)

        if len(themes) > 0:
            default_theme = self.getDefaultThemeName()
            return getattr(theme_container, default_theme, None)


    security.declarePublic('getDefaultThemeName')
    def getDefaultThemeName(self, REQUEST=None):
        """ gets the default theme
        """
 
        themes =  self.getGlobalThemes() 
        for theme in themes:
           if getattr(theme, 'default', None):
               if theme.default is not None:
                   return theme.getId()
        if len(themes) > 0:
           return themes[0].getId()


    security.declareProtected('Manage portal', 'setDefaultTheme')
    def setDefaultTheme(self, default_theme=None, REQUEST=None):
        """ sets the default theme. 
        """
 
        themes =  self.getGlobalThemes() 
        for theme in themes:
             if theme.getId() == default_theme:
                theme.default = 1
             else:
                theme.default = 0
 

    security.declarePublic('getRAMCache')
    def getRAMCache(self, REQUEST=None):
        """ returns the RAM cache
        """

        portal = self.portal_url.getPortalObject()
        if hasattr(portal, CPSSKINS_RAMCACHE_ID):
           return portal[CPSSKINS_RAMCACHE_ID]




    security.declareProtected('Manage portal', 'activateRAMCache')
    def activateRAMCache(self, REQUEST=None):
        """ Activate the RAM cache
        """

        portal = self.portal_url.getPortalObject()
        skinstool = getToolByName(self, 'portal_skins')
                
        custom_skin = skinstool.custom
        CPSSkins_skin = skinstool.CPSSkins

        if CPSSKINS_RENDER_METHOD_ID in custom_skin.objectIds():
            custom_skin._delObject(CPSSKINS_RENDER_METHOD_ID)
                
        if CPSSKINS_RAMCACHE_ID in portal.objectIds():
            portal._delObject(CPSSKINS_RAMCACHE_ID)
                
        portal.manage_addProduct['StandardCacheManagers'].manage_addRAMCacheManager(CPSSKINS_RAMCACHE_ID)
               
        settings = { 'threshold': '100', 
               'cleanup_interval': '500', 
               'request_vars' : ('AUTHENTICATED_USER', ), 
               'max_age':'1000'} 
                
        templetcache = getattr(portal, CPSSKINS_RAMCACHE_ID)
        templetcache.manage_editProps(title='Templet RAM cache', \
                settings = settings)
             
        CPSSkins_skin[CPSSKINS_RENDER_METHOD_ID].manage_doCustomize(folder_path='custom')    
        render_method = getattr(custom_skin, CPSSKINS_RENDER_METHOD_ID, None)
        render_method.ZCacheable_setManagerId(manager_id = CPSSKINS_RAMCACHE_ID)



    security.declareProtected('Manage portal', 'deactivateRAMCache')
    def deactivateRAMCache(self, REQUEST=None):
        """ Deactivate the RAM cache
        """

        portal = self.portal_url.getPortalObject()
        skinstool = getToolByName(self, 'portal_skins')
                
        custom_skin = skinstool.custom
        CPSSkins_skin = skinstool.CPSSkins

        if CPSSKINS_RENDER_METHOD_ID in custom_skin.objectIds():
            custom_skin._delObject(CPSSKINS_RENDER_METHOD_ID)
                
        if CPSSKINS_RAMCACHE_ID in portal.objectIds():
            portal._delObject(CPSSKINS_RAMCACHE_ID)



    security.declarePublic('getRequestedThemeName')
    def getRequestedThemeName(self, REQUEST=None):
        """Gets the name of the requested theme by checking 
           if there is a 'cpsskins_theme' cookie, ?pp=1, or ?theme=...
           in this order: cookie > theme > pp
        """

        if REQUEST is None:
            return None
        FORM = REQUEST.form
        COOKIES = REQUEST.cookies

        theme = self.getDefaultThemeName()

        if hasattr(REQUEST, CPSSKINS_THEME_COOKIE_NAME ):
           theme = COOKIES[CPSSKINS_THEME_COOKIE_NAME]

        if hasattr(REQUEST, 'theme' ):
           theme = FORM['theme']

        if hasattr(REQUEST, 'pp' ):
           theme = 'printable'

        return theme


    def getEffectiveThemeContainer(self, theme=None):
        """Gets a theme container by theme name - if available.
        """

        if theme is not None:
            theme_container = self.getThemeContainer(theme=theme)

            # theme explicitly specified
            if theme_container is not None:
                  return theme_container

        default_theme_container = self.getThemeContainer(theme='default')
        return default_theme_container
        

    security.declarePublic('getGlobalThemes')
    def getGlobalThemes(self):
        """Gets the list of global themes ('default', 'printable', ... )
           returns a list of names.
        """
     
        themes_container = self.getThemeContainer(parent=1)
        atool = getToolByName(self, 'portal_actions')
        theme_list = []

        if themes_container:
           theme_list = themes_container.objectValues('Portal Theme')

        return theme_list


    security.declareProtected(View, 'getIconFor')
    def getIconFor(self, category, id):
        """ Returns an action icon - based on CMFActionsIcons
        """
        icons = {}
        actionicons = getToolByName(self, 'portal_actionicons', None)

        if actionicons:
            try:
               iconinfo = actionicons.getActionIcon(category, id)
               icon = icons.setdefault( (category, id), iconinfo )
            except:
               pass
            else:
               return icon


    security.declareProtected('Manage Themes', 'delObject')
    def delObject(self, object=None):
        """ Deletes an object """
 
        if object is None:
           return

        container = object.aq_parent

        mtool = getToolByName(self, 'portal_membership')
        if not mtool.checkPermission('Manage Themes', container):
            raise Unauthorized
        container.manage_delObjects(object.getId())

        return 


    security.declareProtected('Manage Themes', 'addPortalTheme')
    def addPortalTheme(self, **kw):
        """ Creates a new Portal Theme. Returns the Portal Theme's id. """

        
        id = kw.get('id', 'PortalTheme')

        ids = self.objectIds()
        new_id = id
        i = 0
        while 1:
           if ids is None: 
              break
           if new_id not in ids:
              break 
           i = i + 1
           new_id = id + str(i)
 
        cpsskins = self.manage_addProduct['CPSSkins']
        cpsskins.manage_addContent(id=new_id, type='Portal Theme')

        theme = getattr(self, new_id, None)
        theme.createThemeSkeleton()

        pageblock = theme.addPageBlock()
        if pageblock is not None:
           maincontent = pageblock.addTemplet(type_name='Main Content Templet')
        pageblock.edit(maxcols=int(3))
        maincontent.edit(xpos=int(1))
        col1 = pageblock.addCellSizer(cellsizer_xpos=int(0))
        col2 = pageblock.addCellSizer(cellsizer_xpos=int(1))
        col3 = pageblock.addCellSizer(cellsizer_xpos=int(2))
        col1.edit(cellwidth='20%') 
        col2.edit(cellwidth='60%') 
        col3.edit(cellwidth='20%') 
        return theme


    security.declareProtected('Manage Themes', 'importTheme')
    def importTheme(self, file=None, REQUEST=None):
        """ Imports a theme from a .zexp file
            Returns the its id
        """

        if file is None:
            return
 
        themes_container = self.getThemeContainer(parent=1)

        tmp_id = 'tmp'
        new_tmp_id = tmp_id
        tmp_dir = getattr(themes_container, tmp_id, None)

        if tmp_dir is None:
           themes_container.invokeFactory('Theme Folder', id=tmp_id)
        else:
           new_tmp_id = getFreeId(themes_container, try_id=tmp_id)
           themes_container.manage_renameObjects([tmp_id], [new_tmp_id])
 
        tmp_dir = getattr(themes_container, new_tmp_id, None)
        if tmp_dir is None:
           return 

        try:
            QuickImporter.manage_doQuickImport(tmp_dir, file, set_owner=0, leave=0, REQUEST=None)
        except:
            return

        theme_id = tmp_dir.objectIds()[0]
        new_id = getFreeId(themes_container, try_id=theme_id)
        if new_id != theme_id: 
           tmp_dir.manage_renameObjects([theme_id], [new_id])

        cookie = tmp_dir.manage_cutObjects([new_id])
        themes_container.manage_pasteObjects(cookie)
        themes_container.manage_delObjects([tmp_dir.getId(),])
      
        return new_id


    security.declareProtected('Manage Themes', 'manage_rebuild')
    def manage_rebuild(self, REQUEST=None):
        """
        Rebuild this theme
        """

        self.rebuild()
        self.manage_permission('View', (), acquire=1)
        self.manage_permission('Manage Themes', ('Manager', 'Owner', 'ThemeManager'), acquire=0)
        self.reindexObjectSecurity()

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_main')


    security.declareProtected('Manage Themes', 'rebuild')
    def rebuild(self):
        """
        Rebuild all the themes
        """

        for obj in self.objectValues('Portal Theme'):
            obj.rebuild()


InitializeClass(PortalThemesTool)


